@@ -1,6 +1,7 @@
 /*=============================================================================
 Blobby Volley 2
 Copyright (C) 2006 Jonathan Sieber (jonathan_sieber@yahoo.de)
+Copyright (C) 2006 Daniel Knobe (daniel-knobe@web.de)
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,28 +20,20 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 #pragma once
 
-#include "InputManager.h"
-#include "RenderManager.h"
+#include "InputSource.h"
+#include "Global.h"
+
+class InputDevice;
 
 class LocalInputSource : public InputSource
 {
-private:
-	int mPlayer;
-public:
-	LocalInputSource(PlayerSide player)
-		: mPlayer(player)
-	{
-		 InputManager::getSingleton()->beginGame(player);
-	}
-	virtual PlayerInput getInput()
-	{
-		return InputManager::getSingleton()->getGameInput(mPlayer);
-	}
+	public:
+		LocalInputSource(PlayerSide player);
+		~LocalInputSource();
 
-	~LocalInputSource()
-	{
-		RenderManager::getSingleton().setMouseMarker(-6);
-		InputManager::getSingleton()->endGame();           
-	}
+		virtual PlayerInputAbs getNextInput();
+
+	private:
+		InputDevice* mInputDevice;
 };
 
