@@ -1,6 +1,7 @@
 /*=============================================================================
 Blobby Volley 2
 Copyright (C) 2006 Jonathan Sieber (jonathan_sieber@yahoo.de)
+Copyright (C) 2006 Daniel Knobe (daniel-knobe@web.de)
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,11 +20,14 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 #pragma once
 
-#include <iostream>
-#include <cstring>
+#include <string>
+#include <iosfwd>
+#include <stdint.h>
 
 #include "raknet/PacketEnumerations.h"
 #include "raknet/NetworkTypes.h"
+#include "raknet/BitStream.h"
+#include "BlobbyDebug.h"
 
 enum MessageType
 {
@@ -33,16 +37,22 @@ enum MessageType
 	ID_WIN_NOTIFICATION,
 	ID_OPPONENT_DISCONNECTED,
 	ID_BALL_RESET,
-	ID_BALL_GROUND_COLLISION,
-	ID_BALL_PLAYER_COLLISION,
+	ID_COLLISION,
+	ID_CURRENTLY_UNUSED_2,	// was ID_BALL_PLAYER_COLLISION, now handled via ID_COLLISION
 	ID_GAME_READY,
-	ID_ENTER_GAME,
+	ID_ENTER_SERVER,
 	ID_PAUSE,
 	ID_UNPAUSE,
 	ID_BLOBBY_SERVER_PRESENT,
-	ID_OLD_CLIENT,
-	ID_UNKNOWN_CLIENT,
-
+	ID_VERSION_MISMATCH,
+	ID_CURRENTLY_UNUSED,	// this value is to ensure network protocol compatibility between 0.9c and 1.0
+	ID_REPLAY,
+	ID_CHAT_MESSAGE,
+	ID_UPDATE_SCORE,		// no longer used, as ID_PHYSIC_UPDATE also contains the score information, and ID_BALL_RESET also works as a sync point for clocks
+	ID_RULES_CHECKSUM,
+	ID_RULES,
+	ID_SERVER_STATUS,
+	ID_CHALLENGE
 };
 
 // General Information:
@@ -52,7 +62,7 @@ enum MessageType
 // 	to a client, the information has to be converted into the view
 // 	of the client.
 
-// ID_INPUT_UPDATE:
+// ID_INPUT_UPDATE = 63:
 // 	Description:
 // 		This packet is sent from client to server every frame.
 // 		It contains the current input state as three booleans.
@@ -73,6 +83,7 @@ enum MessageType
 // 		ID_PHYSIC_UPDATE
 // 		ID_TIMESTAMP
 // 		timestamp (int)
+//		packet_number (unsigned char)
 // 		Physic data (analysed by PhysicWorld)
 //
 // ID_WIN_NOTIFICATION
@@ -88,33 +99,25 @@ enum MessageType
 // 	Description:
 // 		 Message sent from server to all clients when the ball
 // 		 is reset to the starting position. It includes an information
-// 		 about the current point state.
+// 		 about the current point state and is used to synchronize
+//		 the clocks.
 // 	Structure:
 // 		ID_BALL_RESET
 // 		serving player (PlayerSide)
 // 		left score (int)
 // 		right score (int)
+//		time (int)
 //
-// ID_BALL_GROUND_COLLISION
+// ID_COLLISION
 // 	Description:
 // 		Message sent from server to all clients when the ball
-// 		hits the ground. It is the only valid reason for a whistle
-// 		sound during the game. The side attribute tells on which side
-//		the ball hit the ground.
-// 	Structure:
-// 		ID_BALL_GROUND_COLLISION
-//		side (PlayerSide)
-//
-// ID_BALL_PLAYER_COLLISION
-// 	Description:
-// 		Message sent from server to all clients when the ball
-// 		hits a player.  It is the only valid reason for a player
-// 		collision sound. The player attribute tells which player
-//		hit the ball.
+// 		hits a player or the ground.  It is the only valid reason for a player
+// 		collision sound. The event attribute contains the DuelMatch Event that
+//		caused the packet to be sent, intensity contains the hit intensity (only valid for player collisions)
 // 	Structure:
 // 		ID_BALL_PLAYER_COLLISION
+//		event (int)
 // 		intensity (float)
-//		player (PlayerSide)
 //
 // ID_GAME_READY
 // 	Description:
@@ -124,18 +127,21 @@ enum MessageType
 // 		opponent.
 // 	Structure:
 // 		ID_GAME_READY
-// 		opponentname(char[16])
+//		gamespeed (int)
+// 		opponent name (char[16])
+//		opponent color (int)
 //
-// ID_ENTER_GAME
+// ID_ENTER_SERVER
 // 	Description:
 // 		Message sent from client to server after connecting to it.
 // 		The side attribute tells the server on which side the client
 // 		wants to play. The name attribute reports to players name,
-// 		truncated to 16 characters.
+// 		truncated to 16 characters. Color is the network color.
 // 	Structure:
-// 		ID_ENTER_GAME
+// 		ID_ENTER_SERVER
 // 		side (PlayerSide)
 // 		name (char[16])
+//		color (int)
 //
 // ID_PAUSE
 // 	Description:
@@ -168,38 +174,93 @@ enum MessageType
 // 		major (int)
 // 		minor (int)
 //
-// ID_OLD_CLIENT
+// ID_VERSION_MISMATCH
 // 	Description:
 // 		Sent from server to client if the version number
-// 		is to old or simply missing
+// 		differes from the one of the server.
 // 	Structure:
-// 		ID_OLD_CLIENT
+// 		ID_VERSION_MISMATCH
+//		server_major (int)
+//		server_minor (int)
 //
-// ID_UNKNOWN_CLIENT
+// ID_REPLAY
 // 	Description:
-// 		Sent from server to client if the version number
-// 		is unknown, typically because the client is more
-// 		recent than the server
+// 		Sent from client to server to request a replay
+// 		Sent from server to client to transmitt the replay
 // 	Structure:
-// 		ID_UNKNOWN_CLIENT
+// 		ID_REPLAY
+//		size (int)
+//		data
+//
+// ID_RULES_CHECKSUM
+// 	Description:
+// 		Sent from server to client to tell rules file checksum
+// 		Client should send ID_RULES after receiving ID_RULES_CHECKSUM
+// 			to tell server if he needs rules file transmitting
+// 	Structure:
+// 		ID_RULES_CHECKSUM
+//		checksum (int)
+//
+// ID_RULES
+// 	Description:
+// 		Sent from client to server to request a rules file
+// 		Sent from server to client to transmit the rules file
+// 		Game is starting only after transmitting a rules file
+// 	Structure (from client to server):
+// 		ID_RULES
+//		needRules (bool)
+// 	Structure (from server to client):
+// 		ID_RULES
+//		size (int)
+//		data
+//
+// ID_SERVER_STATUS
+// 	Description:
+//		Sent from server to waiting clients with information about the
+//		current server status
+//	Structure:
+//		ID_SERVER_STATUS
+//		vector<string> playernames
+//
+// ID_CHALLENGE
+// 	Description:
+//		Sent when the client wants to start a game. If desired opponent is set, the server looks for that
+//		opponent and matches these players.
+//		Sent from the server when another player wants to start a game with this client.
+//	Structure:
+//		ID_CHALLENGE
+//		PlayerID opponent
 //
 
-class UserConfig;
 
-struct ServerInfo
+class IUserConfigReader;
+
+struct ServerInfo : public ObjectCounter<ServerInfo>
 {
-	ServerInfo(RakNet::BitStream& stream, const char* ip);
-	ServerInfo(UserConfig& config);
+	// read server info from a bit stream, additionally, the server address and port are needed
+	ServerInfo(RakNet::BitStream& stream, const char* ip, uint16_t port);
+	// read server info from a user config object
+	ServerInfo(const IUserConfigReader& config);
 	ServerInfo(const std::string& playername);
-	ServerInfo() {}
 
 	void writeToBitstream(RakNet::BitStream& stream);
+
+	/// \todo maybe we should define ServerInfo a little bit more
+	///			as e.g., hostname can be left uninitialised on server
+	/// we combine to functionsalities here: server information and server addresses.
 	int activegames;
+	int gamespeed;
+	uint16_t port;
 	char hostname[64];
 	char name[32];
-	char waitingplayer[64];
+	int waitingplayers;
 	char description[192];
+
+	char rulestitle[32];
+	char rulesauthor[32];
+
+	static const size_t BLOBBY_SERVER_PRESENT_PACKET_SIZE;
 };
 
 bool operator == (const ServerInfo& lval, const ServerInfo& rval);
-
+std::ostream& operator<<(std::ostream& stream, const ServerInfo& val);
