@@ -1,6 +1,7 @@
 /*=============================================================================
 Blobby Volley 2
 Copyright (C) 2006 Jonathan Sieber (jonathan_sieber@yahoo.de)
+Copyright (C) 2006 Daniel Knobe (daniel-knobe@web.de)
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -17,34 +18,42 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 =============================================================================*/
 
+/* header include */
+#include "SoundManager.h"
+
+/* includes */
 #include <iostream>
 #include <cassert>
-#include <physfs.h> 
-#include "SoundManager.h"
-#include "Global.h"
 
+#include "Global.h"
+#include "FileRead.h"
+
+/* implementation */
 SoundManager* SoundManager::mSingleton;
 
 Sound* SoundManager::loadSound(const std::string& filename)
 {
-	PHYSFS_file* fileHandle = PHYSFS_openRead(filename.c_str());
-	if (!fileHandle)
-		throw FileLoadException(std::string(filename));
-	int fileLength = PHYSFS_fileLength(fileHandle);
-	PHYSFS_uint8* fileBuffer = 
-		new PHYSFS_uint8[fileLength];
-	PHYSFS_read(fileHandle, fileBuffer, 1, fileLength);
-	SDL_RWops* rwops = SDL_RWFromMem(fileBuffer, fileLength);
+	FileRead file(filename);
+	int fileLength = file.length();
+
+	// safe file data into a shared_array to ensure it is deleten properly
+	// in case of exceptions
+	boost::shared_array<char> fileBuffer = file.readRawBytes( fileLength );
+
+	SDL_RWops* rwops = SDL_RWFromMem(fileBuffer.get(), fileLength);
+
+	// we don't need the file handle anymore
+	file.close();
 
 	SDL_AudioSpec newSoundSpec;
 	Uint8* newSoundBuffer;
 	Uint32 newSoundLength;
 	if (!SDL_LoadWAV_RW(rwops , 1,
 			&newSoundSpec, &newSoundBuffer, &newSoundLength))
-		throw FileLoadException(filename);
-	delete[] fileBuffer;
-	PHYSFS_close(fileHandle);
+		BOOST_THROW_EXCEPTION(FileLoadException(filename));
 
+
+	// check if current audio format is target format
 	if (newSoundSpec.freq == mAudioSpec.freq &&
 		newSoundSpec.format == mAudioSpec.format &&
 		newSoundSpec.channels == mAudioSpec.channels)
@@ -57,35 +66,39 @@ Sound* SoundManager::loadSound(const std::string& filename)
 		SDL_FreeWAV(newSoundBuffer);
                 return newSound;
 	}
-	else
+	else	// otherwise, convert audio
 	{
 		SDL_AudioCVT conversionStructure;
 		if (!SDL_BuildAudioCVT(&conversionStructure,
 			newSoundSpec.format, newSoundSpec.channels, newSoundSpec.freq,
 			mAudioSpec.format, mAudioSpec.channels, mAudioSpec.freq))
-				throw FileLoadException(filename);
-		conversionStructure.buf = 
+				BOOST_THROW_EXCEPTION ( FileLoadException(filename) );
+		conversionStructure.buf =
 			new Uint8[newSoundLength * conversionStructure.len_mult];
 		memcpy(conversionStructure.buf, newSoundBuffer, newSoundLength);
 		conversionStructure.len = newSoundLength;
 
 		if (SDL_ConvertAudio(&conversionStructure))
-			throw FileLoadException(filename);
+			BOOST_THROW_EXCEPTION ( FileLoadException(filename) );
 		SDL_FreeWAV(newSoundBuffer);
 
 		Sound *newSound = new Sound;
 		newSound->data = conversionStructure.buf;
 		newSound->length = Uint32(conversionStructure.len_cvt);
-		newSound->position = 0;	
+		newSound->position = 0;
 		return newSound;
 	}
 }
 
-
 bool SoundManager::playSound(const std::string& filename, float volume)
 {
 	if (!mInitialised)
 		return false;
+
+	// everything is fine, so we return true
+	// but we don't need to play the sound
+	if( mMute )
+		return true;
 	try
 	{
 		Sound* soundBuffer = mSound[filename];
@@ -95,16 +108,15 @@ bool SoundManager::playSound(const std::string& filename, float volume)
 			mSound[filename] = soundBuffer;
 		}
 		Sound soundInstance = Sound(*soundBuffer);
-		soundInstance.volume = 
+		soundInstance.volume =
 			volume > 0.0 ? (volume < 1.0 ? volume : 1.0) : 0.0;
-		SDL_LockAudio();
+		SDL_LockAudioDevice(mAudioDevice);
 		mPlayingSound.push_back(soundInstance);
-		SDL_UnlockAudio();
+		SDL_UnlockAudioDevice(mAudioDevice);
 	}
-	catch (FileLoadException exception)
+	catch (const FileLoadException& exception)
 	{
-		std::cerr << "Warning: Couldn't load "
-			<< exception.filename << " !" << std::endl;
+		std::cerr << "Warning: " << exception.what() << std::endl;
 		return false;
 	}
 	return true;
@@ -120,44 +132,50 @@ bool SoundManager::init()
 	desiredSpec.callback = playCallback;
 	desiredSpec.userdata = mSingleton;
 
-	
-	if (SDL_OpenAudio(&desiredSpec, &mAudioSpec))
+	mAudioDevice = SDL_OpenAudioDevice(NULL, 0, &desiredSpec, &mAudioSpec, SDL_AUDIO_ALLOW_FORMAT_CHANGE);
+
+	if (desiredSpec.format != mAudioSpec.format)
 	{
-		std::cerr << "Warning: Couldn't open audio Device!"
-			<< std::endl;
+		std::cerr << "Warning: Can't create device with desired audio spec!" << std::endl;
+		std::cerr << "Reason: " << SDL_GetError() << std::endl;
+	}
+
+	if (mAudioDevice == 0)
+	{
+		std::cerr << "Warning: Couldn't open audio Device!" << std::endl;
 		std::cerr << "Reason: " << SDL_GetError() << std::endl;
 		return false;
 	}
-	SDL_PauseAudio(0);
+
+	SDL_PauseAudioDevice(mAudioDevice, 0);
 	mInitialised = true;
 	mVolume = 1.0;
 	return true;
 }
 
 void SoundManager::playCallback(void* singleton, Uint8* stream, int length)
-{	
-	std::list<Sound>& playingSound = 
-		((SoundManager*)singleton)->mPlayingSound;
-	int volume = int(SDL_MIX_MAXVOLUME * 
-		((SoundManager*)singleton)->mVolume);
-	for (std::list<Sound>::iterator iter = playingSound.begin();
-		       iter != playingSound.end(); ++iter)
+{
+	SDL_memset(stream, 0, length);
+	std::list<Sound>& playingSound = ((SoundManager*)singleton)->mPlayingSound;
+	int volume = int(SDL_MIX_MAXVOLUME * ((SoundManager*)singleton)->mVolume);
+
+	for (auto iter = playingSound.begin(); iter != playingSound.end(); ++iter)
 	{
-		int bytesLeft = iter->length
-			- iter->position;
+		int bytesLeft = iter->length - iter->position;
 		if (bytesLeft < length)
 		{
-			SDL_MixAudio(stream, iter->data + iter->position,
-				bytesLeft, int(volume * iter->volume));
-			std::list<Sound>::iterator eraseIter = iter;
-			std::list<Sound>::iterator nextIter = ++iter;
+			SDL_MixAudio(stream, iter->data + iter->position, bytesLeft, int(volume * iter->volume));
+			auto eraseIter = iter;
+			auto nextIter = ++iter;
 			playingSound.erase(eraseIter);
 			iter = nextIter;
+			// prevents increment of past-end-interator
+			if(iter == playingSound.end())
+				break;
 		}
 		else
 		{
-			SDL_MixAudio(stream, iter->data + iter->position,
-				length, int(volume * iter->volume));
+			SDL_MixAudioFormat(stream, iter->data + iter->position, mSingleton->mAudioSpec.format, length, int(volume * iter->volume));
 			iter->position += length;
 		}
 	}
@@ -177,7 +195,8 @@ void SoundManager::deinit()
 			delete iter->second;
 		}
 	}
-	SDL_CloseAudio();
+	SDL_UnlockAudioDevice(mAudioDevice);
+	SDL_CloseAudioDevice(mAudioDevice);
 	mInitialised = false;
 }
 
@@ -188,8 +207,10 @@ SoundManager* SoundManager::createSoundManager()
 
 SoundManager::SoundManager()
 {
+	mMute = false;
 	mSingleton = this;
 	mInitialised = false;
+	mAudioDevice = 0;
 }
 
 SoundManager::~SoundManager()
@@ -214,20 +235,27 @@ void SoundManager::setVolume(float volume)
 
 void SoundManager::setMute(bool mute)
 {
+	// don't do anything if mute is set.
+	// this prevents the crash under xp when mute is set to false, as the second call to SDL_PauseAudio(false)
+	// never returns in that case.
+	if( mute == mMute )
+		return;
+
 	static bool locked = false;
 	if (mute)
 	{
 		if (!locked)  //locking audio twice leads to a bug(??)
 		{
 			locked = true;
-			SDL_LockAudio();
+			SDL_LockAudioDevice(mAudioDevice);
 		}
 	}
 	else
 	{
 		mPlayingSound.clear();
-		SDL_UnlockAudio();
+		SDL_UnlockAudioDevice(mAudioDevice);
 		locked = false;
 	}
-	SDL_PauseAudio((int)mute);
+	mMute = mute;
+	SDL_PauseAudioDevice(mAudioDevice, (int)mute);
 }
