@@ -1,6 +1,7 @@
 /*=============================================================================
 Blobby Volley 2
 Copyright (C) 2006 Jonathan Sieber (jonathan_sieber@yahoo.de)
+Copyright (C) 2006 Daniel Knobe (daniel-knobe@web.de)
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -17,10 +18,19 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 =============================================================================*/
 
-#include <iostream>
+/* header include */
 #include "SpeedController.h"
 
-#include <SDL/SDL.h>
+/* includes */
+#include <algorithm>
+
+#include <SDL2/SDL.h>
+
+/* implementation */
+/// this is required to reduce rounding errors. now we have a resolution of
+/// 1s. This is much better than SDL_Delay can handle, but we prevent 
+/// accumulation errors.
+const int PRECISION_FACTOR = 1000;
 
 SpeedController* SpeedController::mMainInstance = NULL;
 
@@ -32,6 +42,8 @@ SpeedController::SpeedController(float gameFPS)
 	mFPSCounter = 0;
 	mOldTicks = SDL_GetTicks();
 	mFPS = 0;
+	mBeginSecond = mOldTicks;
+	mCounter = 0;
 }
 
 SpeedController::~SpeedController()
@@ -43,43 +55,55 @@ void SpeedController::setGameSpeed(float fps)
 	if (fps < 5)
 		fps = 5;
 	mGameFPS = fps;
+	
+	/// \todo maybe we should reset only if speed changed?
+	mBeginSecond = mOldTicks;
+	mCounter = 0;
 }
 
-bool SpeedController::doFramedrop()
+bool SpeedController::doFramedrop() const
 {
 	return mFramedrop;
 }
 
 void SpeedController::update()
 {
-	mFramedrop = false;
-	int rateTicks = std::max(int(1000 / mGameFPS), 1);
+	int rateTicks = std::max( static_cast<int>(PRECISION_FACTOR * 1000 / mGameFPS), 1);
+	
 	static int lastTicks = SDL_GetTicks();
-	static int lastDrawnFrame = lastTicks;
-	static unsigned int beginSecond = lastTicks;
-	static int counter = 0;
 
-	if (counter == mGameFPS)
+	if (mCounter == mGameFPS)
 	{
-		const int delta = SDL_GetTicks() - beginSecond;
+		const int delta = SDL_GetTicks() - mBeginSecond;
 		int wait = 1000 - delta;
 		if (wait > 0)
 			SDL_Delay(wait);
 	}
-	if (beginSecond + 1000 <= SDL_GetTicks())
+	if (mBeginSecond + 1000 <= SDL_GetTicks())
 	{
-		beginSecond = SDL_GetTicks();
-		counter = 0;
+		mBeginSecond = SDL_GetTicks();
+		mCounter = 0;
 	}
 
-	const int delta = SDL_GetTicks() - beginSecond;
-	if (delta / rateTicks <= counter)
+	const int delta = SDL_GetTicks() - mBeginSecond;
+	if ( (PRECISION_FACTOR * delta) / rateTicks <= mCounter)
 	{
-		int wait = ((counter+1)*rateTicks) - delta;
+		int wait = ((mCounter+1)*rateTicks/PRECISION_FACTOR) - delta;
 		if (wait > 0)
 			SDL_Delay(wait);
 	}
-	counter++;
+	
+	// do we need framedrop?
+	// if passed time > time when we should have drawn next frame
+	// maybe we should limit the number of consecutive framedrops?
+	// for now: we can't do a framedrop if we did a framedrop last frame
+	if ( delta * PRECISION_FACTOR > rateTicks * (mCounter + 1) && !mFramedrop)
+	{
+		mFramedrop = true;
+	} else
+		mFramedrop = false;
+
+	mCounter++;
 
 	//calculate the FPS of drawn frames:
 	if (mDrawFPS)
@@ -97,8 +121,6 @@ void SpeedController::update()
 
 	//update for next call:
 	lastTicks = SDL_GetTicks();
-	if (!mFramedrop)
-		lastDrawnFrame = lastTicks;
 }
 
 
