@@ -1,6 +1,7 @@
 /*=============================================================================
 Blobby Volley 2
-Copyright (C) 2008 Jonathan Sieber (jonathan_sieber@yahoo.de)
+Copyright (C) 2006 Jonathan Sieber (jonathan_sieber@yahoo.de)
+Copyright (C) 2006 Daniel Knobe (daniel-knobe@web.de)
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,28 +20,28 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 #pragma once
 
-#include "Player.h"
-#include "State.h"
+#include "GameState.h"
 
-class LocalGameState : public State
+#include <boost/scoped_ptr.hpp>
+
+class ReplayRecorder;
+
+/*! \class LocalGameState
+	\brief state for singleplayer game
+*/
+class LocalGameState : public GameState
 {
-private:
-	Player mLeftPlayer;
-	Player mRightPlayer;
-	
-	bool mLeftOscillate;
-	bool mRightOscillate;
-	bool mPaused;
-	bool mSaveReplay;
-	bool mWinner;
-	std::string mFilename;
-	
-	DuelMatch* mMatch;
-	ReplayRecorder* mRecorder;
-public:
-	LocalGameState();
-	virtual ~LocalGameState();
-	virtual void step();
-	
+	public:
+		LocalGameState();
+		virtual ~LocalGameState();
+
+		// implementation of the State Interface
+		virtual void step_impl();
+		virtual const char* getStateName() const;
+
+	private:
+		bool mWinner;
+
+		boost::scoped_ptr<ReplayRecorder> mRecorder;
 };
 
